﻿using System.Collections.Generic;
using Newtonsoft.Json;

namespace Jsdl.Domain
{
	public enum OperationTransports
	{
		get,
		post,
		put,
		delete
	}

	// TODO default: json when POST; query when GET
	public enum OperationEnvelope
	{
		json, 
		query, 
		url
	}

	public class Operation
	{
		public Operation()
		{
			Parameters = new List<ServiceParameter>();
		}

		[JsonProperty(PropertyName = "transport")]
		public OperationTransports Transport { get; set; }

		[JsonProperty(PropertyName = "target")]
		public string Target { get; set; }

		[JsonProperty(PropertyName = "description")]
		public string Description { get; set; }

		[JsonProperty(PropertyName = "envelope")]
		public OperationEnvelope Envelope { get; set; }

		public OperationEnvelope ActualEnvelope
		{
			get
			{
				if (Transport == OperationTransports.get)
					return OperationEnvelope.query;
				return Envelope;
			}
		}

		[JsonProperty(PropertyName = "contentType")]
		public string ContentTypeRaw { get; set; }

		[JsonIgnore]
		public string ContentType
		{
			get { return ContentTypeRaw ?? "application/json"; }
		}

		[JsonProperty(PropertyName = "parameters")]
		public List<ServiceParameter> Parameters { get; set; }

		[JsonProperty(PropertyName = "parameter")]
		public ServiceParameter Parameter { get; set; }

		[JsonProperty(PropertyName = "returns")]
		public ServiceType Returns { get; set; }

		[JsonProperty(PropertyName = "maxAge")]
		public int? MaxAgeRaw { get; set; }
	
		[JsonIgnore]
		public int? MaxAge
		{
			get { return MaxAgeRaw.HasValue ? MaxAgeRaw.Value : 0; }
		}
	}
}