﻿using System;
using System.Collections.Generic;
using System.IO;

namespace Jsdl
{
	class Program
	{
		static void Main(string[] args)
		{
			// FOR TESTING ONLY
			//var json = File.ReadAllText(@"Path/To/Json");
			//var gen = CodeGenerator.Create("Csharp");
			//var code = gen.Generate(json);

			if (args.Length > 0)
			{
				ParseArguments(args);

				var outputType = GetArgument("--type", "Csharp");
				var className = GetArgument("--class", "UnknownServiceClient");
				var namespaceName = GetArgument("--namespace", "UnknownNamespace");

				var outputFile = GetArgument("--output", null);
				var jsdl = GetArgument("--jsdl", null);

				if (jsdl != null && outputFile != null)
				{
					var generator = CodeGenerator.Create(outputType);

					foreach (var pair in arguments)
					{
						var prop = generator.GetType().GetProperty(pair.Key.Substring(2));
						if (prop != null)
							prop.SetValue(generator, ConvertValue(pair.Value, prop.PropertyType)); 
					}

					try
					{
						var output = jsdl.StartsWith("http://") || jsdl.StartsWith("https://")
							? generator.GenerateFromUrl(jsdl, className, namespaceName)
							: generator.Generate(File.ReadAllText(jsdl), className, namespaceName);

						File.WriteAllText(outputFile, output);
					}
					catch (Exception ex) 
					{
						Console.WriteLine("Exception: " + ex.Message);
					}
				}
				else
					Console.WriteLine("Exception: '--jsdl' and '--output' argument must be set!");
			}
			else
			{
				Console.WriteLine("Arguments: ");

				Console.WriteLine("--jsdl [URL or path]");
				Console.WriteLine("--output [path]");
				Console.WriteLine("--type (TypeScript|Csharp), default: Csharp");

				Console.WriteLine("--class [web service client class name], default: UnknownServiceClient");
				Console.WriteLine("--type [web service client class namespace], default: UnknownNamespace");

				Console.WriteLine();
				Console.WriteLine();

				Console.WriteLine("TypeScript arguments");
				Console.WriteLine("--GenerateServiceClass (true|false), default: false");
				Console.WriteLine("--GenerateBaseUrl (true|false), default: true");
				Console.WriteLine("--GenerateInterfaces (true|false), default: true");
				Console.WriteLine("--UseKnockoutJs (true|false), default: true");
				Console.WriteLine("--LibraryPath [path to typescript libraries], default: libs");

				Console.WriteLine();
				Console.WriteLine("Csharp arguments");
				Console.WriteLine("--UseMyToolkit (true|false), default: false");
			}
		}

		private static object ConvertValue(string value, Type type)
		{
			if (type == typeof(string))
				return value;
			if (type == typeof(bool))
				return bool.Parse(value);
			if (type == typeof(int))
				return int.Parse(value);
			if (type == typeof(double))
				return double.Parse(value);
			if (type == typeof(decimal))
				return decimal.Parse(value);
			if (type == typeof(float))
				return float.Parse(value); 

			throw new NotImplementedException("Could not convert argument to desired type");
		}

		static readonly Dictionary<string, string> arguments = new Dictionary<string, string>();
		static void ParseArguments(string[] args)
		{
			string key = null; 
			foreach (var arg in args)
			{
				if (key == null)
					key = arg;
				else
				{
					arguments.Add(key, arg);
					key = null; 
				}
			}
		}

		static string GetArgument(string key, string defaultValue)
		{
			if (arguments.ContainsKey(key))
				return arguments[key];
			return defaultValue;
		}
	}
}
